//==================================================================================================
// Full Demonstration for Step CYC10 board.
// 
// Using Ctrl-U to compile and upload this sketch,
// then use Ctrl-Shift-M to open Serial Monitor
//
// This sketch will demo:
// 1) Update/refresh the 7 segment display through system timer
// 2) Receive message on UART. Users can send message through the input box at the top of 
//    the Serial Monitor. And such message will be displayed on Serial Monitor
// 3) 5 way navigation switch. Press of the navigation switch will be displayed on Serial
//    Monitor
// 4) DIP Switch will be read by the processor, and reflected on the LED
// 5) Read X-Y-Z value from the APDS9901 sensor (3-axis accelerometer). And if the user moves
//    the board, such move will be detected by the APDS9901 and report as an activity trigger.
//    When software sees the activity trigger, it will light up the REB LED on the board.
//==================================================================================================




#include "Step_CYC10_I2C.h"

#define APDS9901_I2C_ADDR              0x39
#define APDS9901_DEVICE_ID             0x20


//----------------------------------------------------------------------------
// Read_APDS9901_WordData()
//
// Parameters:
//      None
//
// Return Value:
//      return interrupt source of APDS9901
//
// Remarks:
//      Function to read APDS9901_INT_SOURCE
//----------------------------------------------------------------------------

uint16_t Read_APDS9901_WordData(uint8_t reg)
{
    uint8_t buf[2];
    uint16_t temp;
      
    I2C.masterRead(APDS9901_I2C_ADDR, 0xA0|reg, buf, 2);  
    temp = ((uint16_t)buf[1] << 8) | buf[0] ;
    return temp;
      
} // End of Read_APDS9901_WordData()

//----------------------------------------------------------------------------
// APDS9901_init()
//----------------------------------------------------------------------------

void APDS9901_init()
{
    
    uint8_t t;
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x00, 0x00);
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x01, 0xFF);
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x02, 0xFF);
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x03, 0xFF);
    
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x0E, 0x01);
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x0F, 0x20);
    I2C.masterWrite(APDS9901_I2C_ADDR, 0x00, 0x0F);
    delay(12);
    
    I2C.masterRead(APDS9901_I2C_ADDR, 0x12, &t, 1);
    if (t != APDS9901_DEVICE_ID) {
        Serial.println ("APDS9901 device ID unrecognized!!!!!");
    }
 //   Read_APDS9901_WordData();
    
} // End of APDS9901_init()


void Get_APDS9901_Data(uint16_t* ch0_data,uint16_t* ch1_data,uint16_t* prox_data)
{
    uint8_t t;
    
    I2C.masterRead(APDS9901_I2C_ADDR, 0x12, &t, 1);
    if (t != APDS9901_DEVICE_ID) {
        Serial.println ("APDS9901 device ID unrecognized!!!!!");
        Serial.println(t, HEX);
        while(1);
    }  
    *ch0_data = Read_APDS9901_WordData(0x14);
    *ch1_data = Read_APDS9901_WordData(0x16);
    *ch1_data = Read_APDS9901_WordData(0x18);
}

//================================================================================================================
// UART receive
//================================================================================================================

uint8_t uart_rx_buf [256] = {0};
uint8_t uart_rx_index_write_point = 0;
uint8_t uart_rx_index_read_point = 0;

void uart_rx_isr()
{
    uint8_t t;
    
    t = Serial.read();    
    uart_rx_buf [uart_rx_index_write_point++] = t;
    
} // End of uart_rx_isr()



//================================================================================================================
// 5 way navigation switch
//================================================================================================================

uint8_t keys[256] ={0};
uint8_t key_write_point = 0;
uint8_t key_read_point = 0;

void int0_keys_isr()
{
    uint8_t t;

    t = GPIO_P0 & 0x1F;
    if (t) {
        keys[key_write_point++] = t;
    }
} // End of int0_keys_isr()


//================================================================================================================
// setup()
//================================================================================================================

void setup() {
    delay(1000);
    Serial.print ("init");
    APDS9901_init();
   
    attachInterrupt (INT_UART_RX_INDEX, uart_rx_isr, RISING);
    attachInterrupt (INT_EXTERNAL_1ST, int0_keys_isr, RISING);

    interrupts();

 //   GPIO_P2 = 0xAC;
 //   GPIO_P3 = 0xAA;
 //   delay(5000);

} // End of setup()


//================================================================================================================
// loop()
//================================================================================================================

void loop() {

    static int i = 0;

    uint16_t ch0_data,ch1_data,prox_data;
    
    uint8_t k, t, j;
  
    Serial.print (++i);
    Serial.print (": ");
    
    Serial.print("SW = 0x");
    k = GPIO_P1;

    t = 0;
    for (j = 0; j < 8; ++j) {
        t += ((k >> j) & 1) << (7 - j);
    }
    
    Serial.print(t, HEX);
    Serial.print(" ========== ");
    
    if (uart_rx_index_read_point != uart_rx_index_write_point) {
        Serial.print("\n Got Message: ");
        do {
            Serial.write(&uart_rx_buf [uart_rx_index_read_point++], 1);
        } while(uart_rx_index_read_point != uart_rx_index_write_point);
        Serial.print("\n");
    }


    if (key_read_point != key_write_point) {
        Serial.print("\n Key Pressed: ");
        do {
            k = keys[key_read_point++];

            if (k == 0x1) {
                Serial.println(" Left");
            } else if (k == 0x2) {
                Serial.println(" Center");  
            } else if (k == 0x4) {
                Serial.println(" Down");  
            } else if (k == 0x8) {
                Serial.println(" Up");  
            } else {
                Serial.println(" Right");  
            }

            Serial.println(" ");
            
        } while(key_read_point != key_write_point);
    }

    // RGB LED
    GPIO_P2 = 0xFF;

    // LED / DIP Switch
    GPIO_P3 = ~GPIO_P1;

    Get_APDS9901_Data(&ch0_data,&ch1_data,&prox_data);
   
    Serial.print ("ch0_data = ");
    Serial.print (ch0_data);
    
    Serial.print (", ch1_data = ");
    Serial.print (ch1_data);
    
    Serial.print (", prox_data = ");
    Serial.println (prox_data);
    Serial.print ("HELLO!");
    delay(1000);

} // End of loop()